clc;
close all;
clear all;

global N; 
global Q; 
global R; 
global x0; 
global xref;

Nx = 4;     % number of differential states
Nu = 1;     % number of control inputs
Nz = Nx+Nu;
N = 30;     % number of shooting intervals
T = 1.5;    % Horizon length
Ts = T/N;
x0 = [0; pi; 0; 0];     % initial value
xref = [0; 0; 0; 0];    % reference value
Q = diag([0 0 0 0]);    % no stage cost on states
R = 1;
umax = 20;

global input;
input.Ts = Ts;
input.nSteps = 1;
input.sens = 1;

% Bound values for controls:
LB_SS = -umax*ones(N*Nu,1);
UB_SS = umax*ones(N*Nu,1);

% Initialize single shooting variables:
Z_SS = zeros(Nu,N);

% Setting options and solving the problem with fmincon + SINGLE SHOOTING:
options = optimoptions(@fmincon,'Display','iter','Algorithm','interior-point','GradObj','on','GradConstr','on','Hessian','user-supplied','HessFcn',@hessian_single);
sol_single = fmincon(@cost_single,Z_SS,[],[],[],[],LB_SS,UB_SS,@constr_single,options);
Z_SS = sol_single;
U = Z_SS;

% Recover the full state trajectory by simulation:
X = zeros(Nx,N+1);
X(:,1) = x0;
for k = 1:N
    u_k = U(:,k);
    input.x = X(:,k);
    input.u = u_k;
    output = RK4_integrator( @ode, input );
    x_k = output.value;
    X(:,k+1) = x_k;
end

% Plot results single (and multiple) shooting method:
figure;
subplot(321);
plot([0:N]*Ts,X(1,:),'--rx'); hold on;
% plot([0:N]*Ts,X2(1,:),'--bo');
xlabel('time(s)'); ylabel('p');
% legend('Single shooting', 'Multiple shooting')

subplot(322);
plot([0:N]*Ts,X(2,:),'--rx'); hold on;
% plot([0:N]*Ts,X2(2,:),'--bo');
xlabel('time(s)'); ylabel('\theta');
% legend('Single shooting', 'Multiple shooting')

subplot(323);
plot([0:N]*Ts,X(3,:),'--rx'); hold on;
% plot([0:N]*Ts,X2(3,:),'--bo');
xlabel('time(s)'); ylabel('v');
% legend('Single shooting', 'Multiple shooting')

subplot(324);
plot([0:N]*Ts,X(4,:),'--rx'); hold on;
% plot([0:N]*Ts,X2(4,:),'--bo');
xlabel('time(s)'); ylabel('\omega');
% legend('Single shooting', 'Multiple shooting')

subplot(3,2,[5 6]);
stairs([0:N-1]*Ts,U,'--rx'); hold on;
% stairs([0:N-1]*Ts,U2,'--bo');
xlabel('time(s)'); ylabel('F');
% legend('Single shooting', 'Multiple shooting')


